use std::io::BufReader;
use std::path::PathBuf;
use std::thread;
use std::time::Duration;
extern crate rodio;
mod base;
use base::*;
// use rodio::Source;
use imgui::*;
use walkdir::WalkDir;
mod support;
use env_logger;
use log::*;

fn main() {
    env_logger::init();
    let mut sounds = vec![];

    for entry in WalkDir::new("media")
        .into_iter()
        .filter_map(|e| e.ok())
        .filter(|e| e.path().is_file())
    {
        match Sound::new(&entry.path().to_string_lossy().to_string()) {
            Ok(s) => sounds.push(s),
            Err(e) => error!("{:?} while opening {:?}", e, entry),
        }
    }

    let dev = rodio::default_output_device().unwrap();

    let mut bar = Bar::default();
    bar.bpm = 160;
    bar.repeat = 2;

    let system = support::init(file!());

    system.main_loop(|_, ui| {
        Window::new(im_str!("sources"))
            // .resizable(false)
            .position([400.0, 140.0], Condition::Appearing)
            .collapsible(false)
            // .always_auto_resize(true)
            .size([300.0, 300.0], Condition::FirstUseEver)
            .build(ui, || {
                // let mut s = ui.clone_style();
                // ui.show_style_editor(&mut s);

                ui.tree_node(im_str!("Tree")).build(|| {
                    for s in &sounds {
                        ui.tree_node(&im_str!("{}", s.name)).build(|| {
                            ui.text(im_str!("blah blah"));
                            ui.same_line(0.0);
                            if ui.small_button(im_str!("add")) {}
                            ui.same_line(0.0);
                            if ui.small_button(im_str!("play")) {
                                s.play(&dev);
                            }
                        });
                    }
                });
            });

        Window::new(im_str!("Hello world"))
            .size([300.0, 100.0], Condition::FirstUseEver)
            .build(ui, || {
                ui.text(im_str!("bars"));

                for i in 0..bar.length {
                    ui.same_line(0.0);
                    ui.small_button(&im_str!("{}", i));
                }
            });
    });
}
